const InterfaceDefaultOptions =  {
	interfaceConfig: {},
	origin: null,
	beforeSend: new Function,
	dataFilter: null,
	dataPath: null,
	errorCallback: msg => console.warn('错误信息：' + msg),
	
	type: 'get',
	dataType: 'json',
	contentType: 'application/json; charset=utf-8',
	traditional: true,
	jsonp: null,
};


const AjaxDefaultOptions = {
	type: 'GET',
	url: '',
	data: null,
	traditional: true,
	dataType: 'json',
	jsonp: 'callback',
	contentType: 'application/x-www-form-urlencoded; charset=UTF-8',
	cache: true,
	success: new Function,
	error: new Function,
	uploading: new Function,
	beforeSend: new Function,
	dataFilter: null,
	headers: {
		"Accept": "application/json, text/javascript, */*; q=0.01",
		"Content-Type": "application/json; charset=UTF-8",
		"X-Requested-With": "XMLHttpRequest"
	}
};

let counter = 0;

function loadUrlString(opt) {
	if (opt.url) return opt.url
	if (opt.origin) {
		if (!opt.origin.match(/\/$/) && !opt.pathname.match(/^\//)) {
			return opt.origin + '/' + opt.pathname
		}
		return opt.origin + opt.pathname
	}
	else return opt.pathname
}

function setFirstParam(arg, params) {
	if (!_.isObject(arg)) params[_.keys(params)[0]] = arg;
	else if (!_.isFunction(arg)) _.assign(params, arg);
}

function getAjaxConfig(args, interfaceConfig) {
	const opt = interfaceConfig;
	let selfCallback = null, selfErrorCallback = null, params = _.clone(opt.params);
	
	switch (_.size(args)) {
		case 0:
			break
		case 1 : {
			if (_.isFunction(args[0])) selfCallback = args[0];
			else setFirstParam(args[0], params);
			break
		}
		case 2 : {
			if (_.isFunction(args[0]) && _.isFunction(args[1])) {
				selfCallback = args[0];
				selfErrorCallback = args[1];
			} else {
				setFirstParam(args[0], params);
				selfCallback = args[1];
			}
			break
		}
		default : {
			setFirstParam(args[0], params);
			selfCallback = args[1];
			selfErrorCallback = args[2];
			break
		}
	}
	
	if (_.isFunction(opt.paramsFilter)) params = opt.paramsFilter(params);
	if (_.toLower(opt.type) === 'post' && opt.contentType !== 'application/x-www-form-urlencoded') params = JSON.stringify(params);
	
	const selfConfig = {
		url: loadUrlString(opt),
		data: params,
		callback: selfCallback,
		errorCallback: selfErrorCallback,
		_counter: counter++
	};
	
	const mergeDefaults = _.assignWith(_.clone(AjaxDefaultOptions), selfConfig, (tar, src) => src ? src : tar);
	
	const aConf = _.assign(mergeDefaults, _.pick(opt, ['type', 'traditional', 'dataType', 'jsonp', 'beforeSend']));
	
	return _.assignWith(aConf, opt, (tar, src) => tar ? tar : src)
}

function dealResponse(response) {
	const data = response.data || response;
	const status = response.status || data.status;
	const result = {...data, status};
	return _.pickBy(result, val => !_.isUndefined(val))
}

function successProcess(response, options) {
	const res = dealResponse(response);
	if (res.success && res.success !== 'false') {
		const data = res.data || res;
		options.resolve(options.dataPath ? _.get(data, options.dataPath) : data);
	} else {
		const errText = (res.message || res.msg) ? (res.message || res.msg) : '暂无错误信息';
		options.reject(errText, res, _.pick(options, ['name', 'url']));
	}
}

function failProcess(errorInfo, options) {
	let errText = "ajax error";
	let errorType = errorInfo.errorType;
	let info = errorInfo.status;
	if (errorType === "error" && info.status === "404") errText = "404 不可用的路径";
	else if (errorType === "parsererror") {
		if (info.message || info.msg) errText = info.message || info.msg;
		else if (info.response) errText = "JSON返回数据格式异常！";
		else errText = "服务器返回数据无法解析！";
	} else errText = "服务器异常！";
	
	options.reject(`${options.name}：` + errText, errorInfo);
}

function process (target){
	return [successProcess, failProcess][target]
}

function parseParams (a) {
	const ret = {};
	let seg = a.search.replace(/^\?/, '').split('&');
	if (!!seg === false) return {}
	seg = seg.map(e => ({
		key: e.split('=')[0],
		value: decodeURI(e.split('=')[1])
	}));
	seg.forEach(e => {
		const currentKey = e.key;
		let values = [];
		if (_.isUndefined(ret[currentKey])) {
			seg.filter(e => e.key === currentKey).forEach(e => values.push(e.value));
			if (values.length === 1) ret[currentKey] = values[0];
			else ret[currentKey] = values;
		}
	});
	return ret
}

function parseURL(url) {
	const a = document.createElement('a');
	a.href = url;
	
	return {
		source: url,
		query: a.search,
		origin: a.origin || a.href.match(/\w+:\/\/[^\/\\]+/),
		params: parseParams(a),
		hashQuery: (a.hash.match(/(?=\?).*/) && a.hash.match(/(?=\?).*/)[0]) || "",
		hashParams: /.*\?/.test(a.hash) ? parseParams(a) : {},
		protocol: a.protocol.replace(':', ''),
		host: a.hostname,
		port: a.port,
		file: (a.pathname.match(/\/([^\/?#]+)$/i) || ['', ''])[1],
		hash: a.hash.replace('#', ''),
		path: a.pathname.replace(/^([^\/])/, '/$1'),
		relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || ['', ''])[1],
		segments: a.pathname.replace(/^\//, '').split('/'),
		rebuild: () => buildURL({
			url: this.url,
			hash: this.hash,
			origin: this.origin,
			source: this.source,
			path: this.path,
			params: this.params,
		})
	}
}

function buildURL(options) {
	const urlInfo = parseURL(options.url || options.source);
	_.assign(urlInfo, _.omit(options, ['url']));
	urlInfo.path = urlInfo.path.replace(/^(?=[^\/])/, '/');
	
	let paramsStr = '';
	_.forOwn(urlInfo.params, (value, key) => {
		if (value === null || value === undefined) return
		if (!_.isArray(value)) value = [value];
		value.forEach(val => {
			if (_.isObject(val)) val = encodeURIComponent(JSON.stringify(val));
			paramsStr += `${paramsStr ? '&' : ''}${key}=${encodeURIComponent(val)}`;
		});
	});
	
	return urlInfo.origin + urlInfo.path + (paramsStr ? "?" + paramsStr : "") + (urlInfo.hash ? "#" + urlInfo.hash : "")
}

function serialData(data){
	if(_.isString(data)) return data
	if(_.isObject(data)) {
		const queryList = [];
		_.forOwn(data, (value, key) => {
			if(_.isArray(value)) value.forEach(e => queryList.push(key + '=' + encodeURI(e)));
			else queryList.push(key + '=' + encodeURI(value));
		});
		return queryList.join('&')
	}
}

let jsonpCounter = 100;

function jsonp(options) {
	const jsonpCallbackName = options.jsonp + jsonpCounter++;
	let called = false;
	window[jsonpCallbackName] = data => {
		called = true;
		options.success(data);
	};
	
	const script = document.createElement('script');
	script.type = 'text/javascript';
	const params = _.clone(options.data);
	params[options.jsonp] = jsonpCallbackName;
	script.src = buildURL({
		url: options.url,
		params: params
	});
	document.head.appendChild(script);
	script.onload = function () {
		if (!called) {
			options.fail({
				message: 'jsonp 返回结果不合法',
			}, 'parsererror', {
				status: 200,
				readyState: 4,
			});
		}
		delete window[jsonpCallbackName];
		document.head.removeChild(script);
	};
	script.onerror = function () {
		options.fail({
			message: '请求拒绝访问！',
		}, '(failed)net::ERR_CONNECTION_REFUSED', {
			status: null
		});
		document.head.removeChild(script);
	};
}

function getMethod(xhr, options) {
	xhr.open('GET', buildURL({
		url: options.url,
		params: options.data
	}));
	_.mapKeys(options.headers, (value, key) => xhr.setRequestHeader(key, value));
	options.beforeSend(xhr);
	xhr.send();
}

function postMethod(xhr, options) {
	xhr.open('POST', options.url);
	_.mapKeys(options.headers, (value, key) => xhr.setRequestHeader(key, value));
	options.beforeSend(xhr);
	xhr.send(serialData(options.data));
}

function xhr (options) {
	const xhr = new XMLHttpRequest();
	
	xhr.onreadystatechange = function () {
		if (xhr.readyState === 4) {
			if (xhr.status === 200) {
				const data = _.isFunction(options.dataFilter) ? options.dataFilter(xhr.responseText, xhr.responseType) : xhr.responseText;
				if (_.toLower(options.dataType) === 'json' && _.isString(data)) options.success(JSON.parse(data));
				else options.success(data);
			} else options.fail(xhr.responseText, 'error', {status: xhr.status});
		}
	};
	
	xhr.upload.onprogress = evt => {
		const loaded = evt.loaded;
		const total = evt.total;
		const percent = Math.floor(100 * loaded / total);
		options.uploading && options.uploading(percent);
	};
	
	const xhrMethod = {
		GET: getMethod,
		POST: postMethod,
	};
	
	xhrMethod[_.upperCase(options.type)](xhr, options);
}

function beforeAjax(options) {
	if (_.isFunction(options.switch)) options.switch(true);
}

function afterAjax(options) {
	if (_.isFunction(options.switch)) options.switch(false);
}

function doAjax (options) {
	if (beforeAjax(options)) return
	options.success = (data, status) => {
		if (afterAjax(options)) return
		process(0)({data: data, status: status}, options);
	};
	options.fail = (status, errorType, describe) => {
		if (afterAjax(options)) return
		process(1)({status: status, errorType: errorType, describe: describe}, options);
	};
	
	if (_.toLower(options.dataType) === 'jsonp') jsonp(options);
	else xhr(options);
}

function buildInterface(key, proxy) {
	const selfOpt = proxy.$configMap[key];
	const globalOpt = _.assignWith(_.clone(proxy.$globalOptions), InterfaceDefaultOptions, (tar, src) => tar ? tar : src);
	const iConf = _.assign(globalOpt, selfOpt);
	
	const buildProxy = function (params, callback, errorCallback) {
		const options = getAjaxConfig(arguments, iConf);
		const agent = new Promise((resolve, reject) => {
			options.resolve = function () {
				resolve.call(this, ...arguments);
				options.callback && options.callback.call(this, ...arguments);
			};
			options.reject = function () {
				reject.call(this, ...arguments);
				options.errorCallback && options.errorCallback.call(this, ...arguments);
			};
			
			proxy[key].reject = callback => {
				reject();
				callback && callback();
				options.callback = options.errorCallback = _.noop;
			};
		});
		
		agent.catch = ecb => {
			agent.then(null, ecb);
			return agent
		};
		
		agent.switch = (tar, key) => {
			options.switch = val => tar[key] = val;
			return agent
		};
		
		setTimeout(doAjax, 0, options);
		
		return agent
	};
	
	buildProxy.$options = iConf;
	
	buildProxy.url = iConf.url || iConf.origin + iConf.pathname;
	
	return buildProxy
}

class InterfaceProxy {
	constructor(configMap, globalOptions) {
		if (!_.isArray(globalOptions)) globalOptions = [globalOptions || {}];
		
		this.$globalOptions = globalOptions.reduce((pre, cur) => _.assign(_.clone(pre), cur), {});
		
		this.$configMap = configMap;
		
		this.buildURL = buildURL;
		
		_.forOwn(_.cloneDeep(configMap), (value, key) => this[key] = buildInterface(key, this));
	}
}

export default InterfaceProxy;
