import {serialData, buildURL} from "../util"

function getMethod(xhr, options) {
	xhr.open('GET', buildURL({
		url: options.url,
		params: options.data
	}))
	_.mapKeys(options.headers, (value, key) => xhr.setRequestHeader(key, value))
	options.beforeSend(xhr)
	xhr.send()
}

function postMethod(xhr, options) {
	xhr.open('POST', options.url)
	_.mapKeys(options.headers, (value, key) => xhr.setRequestHeader(key, value))
	options.beforeSend(xhr)
	xhr.send(serialData(options.data))
}

export default function (options) {
	const xhr = new XMLHttpRequest()
	
	xhr.onreadystatechange = function () {
		if (xhr.readyState === 4) {
			if (xhr.status === 200) {
				const data = _.isFunction(options.dataFilter) ? options.dataFilter(xhr.responseText, xhr.responseType) : xhr.responseText
				if (_.toLower(options.dataType) === 'json' && _.isString(data)) options.success(JSON.parse(data))
				else options.success(data)
			} else options.fail(xhr.responseText, 'error', {status: xhr.status})
		}
	}
	
	xhr.upload.onprogress = evt => {
		const loaded = evt.loaded
		const total = evt.total
		const percent = Math.floor(100 * loaded / total)
		options.uploading && options.uploading(percent)
	}
	
	const xhrMethod = {
		GET: getMethod,
		POST: postMethod,
	}
	
	xhrMethod[_.upperCase(options.type)](xhr, options)
}