import {AjaxDefaultOptions as defaultOpt} from '../defaultOptions'

let counter = 0

function loadUrlString(opt) {
	if (opt.url) return opt.url
	if (opt.origin) {
		if (!opt.origin.match(/\/$/) && !opt.pathname.match(/^\//)) {
			return opt.origin + '/' + opt.pathname
		}
		return opt.origin + opt.pathname
	}
	else return opt.pathname
}

function setFirstParam(arg, params) {
	if (!_.isObject(arg)) params[_.keys(params)[0]] = arg
	else if (!_.isFunction(arg)) _.assign(params, arg)
}

export function getAjaxConfig(args, interfaceConfig) {
	const opt = interfaceConfig
	let selfCallback = null, selfErrorCallback = null, params = _.clone(opt.params)
	
	switch (_.size(args)) {
		case 0:
			break
		case 1 : {
			if (_.isFunction(args[0])) selfCallback = args[0]
			else setFirstParam(args[0], params)
			break
		}
		case 2 : {
			if (_.isFunction(args[0]) && _.isFunction(args[1])) {
				selfCallback = args[0]
				selfErrorCallback = args[1]
			} else {
				setFirstParam(args[0], params)
				selfCallback = args[1]
			}
			break
		}
		default : {
			setFirstParam(args[0], params)
			selfCallback = args[1]
			selfErrorCallback = args[2]
			break
		}
	}
	
	if (_.isFunction(opt.paramsFilter)) params = opt.paramsFilter(params)
	if (_.toLower(opt.type) === 'post' && opt.contentType !== 'application/x-www-form-urlencoded') params = JSON.stringify(params)
	
	const selfConfig = {
		url: loadUrlString(opt),
		data: params,
		callback: selfCallback,
		errorCallback: selfErrorCallback,
		_counter: counter++
	}
	
	const mergeDefaults = _.assignWith(_.clone(defaultOpt), selfConfig, (tar, src) => src ? src : tar)
	
	const aConf = _.assign(mergeDefaults, _.pick(opt, ['type', 'traditional', 'dataType', 'jsonp', 'beforeSend']))
	
	return _.assignWith(aConf, opt, (tar, src) => tar ? tar : src)
}