function parseParams (a) {
	const ret = {}
	let seg = a.search.replace(/^\?/, '').split('&')
	if (!!seg === false) return {}
	seg = seg.map(e => ({
		key: e.split('=')[0],
		value: decodeURI(e.split('=')[1])
	}))
	seg.forEach(e => {
		const currentKey = e.key
		let values = []
		if (_.isUndefined(ret[currentKey])) {
			seg.filter(e => e.key === currentKey).forEach(e => values.push(e.value))
			if (values.length === 1) ret[currentKey] = values[0]
			else ret[currentKey] = values
		}
	})
	return ret
}

function parseURL(url) {
	const a = document.createElement('a')
	a.href = url
	
	return {
		source: url,
		query: a.search,
		origin: a.origin || a.href.match(/\w+:\/\/[^\/\\]+/),
		params: parseParams(a),
		hashQuery: (a.hash.match(/(?=\?).*/) && a.hash.match(/(?=\?).*/)[0]) || "",
		hashParams: /.*\?/.test(a.hash) ? parseParams(a) : {},
		protocol: a.protocol.replace(':', ''),
		host: a.hostname,
		port: a.port,
		file: (a.pathname.match(/\/([^\/?#]+)$/i) || ['', ''])[1],
		hash: a.hash.replace('#', ''),
		path: a.pathname.replace(/^([^\/])/, '/$1'),
		relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || ['', ''])[1],
		segments: a.pathname.replace(/^\//, '').split('/'),
		rebuild: () => buildURL({
			url: this.url,
			hash: this.hash,
			origin: this.origin,
			source: this.source,
			path: this.path,
			params: this.params,
		})
	}
}

export function buildURL(options) {
	const urlInfo = parseURL(options.url || options.source)
	_.assign(urlInfo, _.omit(options, ['url']))
	urlInfo.path = urlInfo.path.replace(/^(?=[^\/])/, '/')
	
	let paramsStr = ''
	_.forOwn(urlInfo.params, (value, key) => {
		if (value === null || value === undefined) return
		if (!_.isArray(value)) value = [value]
		value.forEach(val => {
			if (_.isObject(val)) val = encodeURIComponent(JSON.stringify(val))
			paramsStr += `${paramsStr ? '&' : ''}${key}=${encodeURIComponent(val)}`
		})
	})
	
	return urlInfo.origin + urlInfo.path + (paramsStr ? "?" + paramsStr : "") + (urlInfo.hash ? "#" + urlInfo.hash : "")
}

export function serialData(data){
	if(_.isString(data)) return data
	if(_.isObject(data)) {
		const queryList = []
		_.forOwn(data, (value, key) => {
			if(_.isArray(value)) value.forEach(e => queryList.push(key + '=' + encodeURI(e)))
			else queryList.push(key + '=' + encodeURI(value))
		})
		return queryList.join('&')
	}
}